// #define DEBUG

#include <iostream>
#include <vector>
#include <queue>
#include <limits>
#ifdef DEBUG
#include <chrono>
#endif // DEBUG

#define ENDL '\n'
#define INF std::numeric_limits<int>::max()-1

namespace {
template<class T>
using vec = std::vector<T>;

template<class T>
using mat = vec<vec<T>>;

auto bfs_step(mat<bool>& adj, std::queue<int>& q, vec<bool>& vis, vec<int>& dist, vec<bool>& other_vis) -> int {
    auto curr = q.front();
    q.pop();

    if (vis[curr]) {
        return -1;
    }

    vis[curr] = true;

    for (auto i = 0; i < adj.size(); ++i) {
        if (!adj[curr][i]) {
            continue;
        }

        dist[i] = std::min(dist[i], dist[curr] + 1);
        if (other_vis[i]) {
            return i;
        }

        q.emplace(i);
    }

    return -1;
}

auto bidirectional_bfs(int s, int t, mat<bool>& adj) {
    auto n = static_cast<int>(adj.size());

    std::queue<int> s_queue, t_queue;
    vec<bool> s_vis, t_vis;
    vec<int> s_dist, t_dist;

    s_vis = t_vis = vec<bool>(n);
    s_dist = t_dist = vec<int>(n, INF);

    s_queue.emplace(s);
    t_queue.emplace(t);

    s_dist[s] = 0;
    t_dist[t] = 0;

    auto bidirectional_step = [&]() {
        auto in = bfs_step(adj, s_queue, s_vis, s_dist, t_vis);
        if (in != -1) {
            return in;
        }

        return bfs_step(adj, t_queue, t_vis, t_dist, s_vis);
    };

    while (!s_queue.empty() && !t_queue.empty()) {
        auto intersection = bidirectional_step();
        if (intersection == -1) {
            continue;
        }

        return s_dist[intersection] + t_dist[intersection];
    }

    return -1;
}
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);
    std::cout.tie(nullptr);

    int n, m;
    std::cin >> n >> m;

    auto adj = mat<bool>(n, vec<bool>(n, true));
    for (auto i = 0; i < n; ++i) {
        adj[i][i] = false;
    }

    int a, b;
    while (m--) {
        std::cin >> a >> b;

        adj[a][b] = false;
        adj[b][a] = false;
    }

    int q;
    std::cin >> q;

#ifdef DEBUG
    auto time_begin = std::chrono::high_resolution_clock::now();
#endif // DEBUG
    while (q--) {
        std::cin >> a >> b;

        std::cout << bidirectional_bfs(a, b, adj) << ENDL;
    }
#ifdef DEBUG
    auto time_end = std::chrono::high_resolution_clock::now();

    auto time_elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(time_end - time_begin).count();
    std::cout << "Time elapsed: " << time_elapsed << "ms" << ENDL;
#endif // DEBUG

    return 0;
}
