// #define DEBUG

#include <iostream>
#include <vector>
#include <queue>
#include <map>
#ifdef DEBUG
#include <chrono>
#endif // DEBUG

#define ENDL '\n'

namespace {
template<class T>
using vec = std::vector<T>;

template<class T>
using mat = vec<vec<T>>;

auto bfs_step(mat<int> const& adj_vec, std::queue<int>& curr_queue, vec<bool>& curr_vis, vec<int>& curr_dist, vec<bool>& other_vis) {
    auto curr_v = curr_queue.front();
    curr_queue.pop();

    if (other_vis[curr_v]) {
        return curr_v;
    }

    curr_vis[curr_v] = true;
    for (auto& next_v : adj_vec[curr_v]) {
        if (curr_vis[next_v]) {
            continue;
        }

        curr_dist[next_v] = curr_dist[curr_v] + 1;
        curr_queue.push(next_v);
    }

    return -1;
}

auto bidirectional_bfs(int s, int t, mat<int> const& adj_vec) {
    auto n = static_cast<int>(adj_vec.size());

    std::queue<std::pair<int, int>> s_queue;
    vec<bool> s_vis(n);
    vec<int> s_dist(n);

    s_queue.emplace(s, 0);

    while (!s_queue.empty()) {
        int curr_v, curr_d;
        std::tie(curr_v, curr_d) = s_queue.front();
        s_queue.pop();

        if (curr_v == t) {
            return curr_d;
        }

        if (s_vis[curr_v]) {
            continue;
        }
        s_vis[curr_v] = true;

        for (auto& next_v : adj_vec[curr_v]) {
            s_queue.emplace(next_v, curr_d + 1);
        }
    }

    return -1;
}
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);
    std::cout.tie(nullptr);

    int n, m;
    std::cin >> n >> m;

#ifdef DEBUG
    auto time_begin = std::chrono::high_resolution_clock::now();
#endif // DEBUG

    mat<int> adj_vec(n);
    {
        mat<bool> missing_edges(n, vec<bool>(n));

        int a, b;
        while (m--) {
            std::cin >> a >> b;

            missing_edges[a][b] = true;
            missing_edges[b][a] = true;
        }

        for (auto i = 0; i < n; ++i) {
            for (auto j = 0; j < n; ++j) {
                if (missing_edges[i][j]) {
                    continue;
                }

                adj_vec[i].emplace_back(j);
            }
        }
    }

    int q;
    std::cin >> q;

    int c, d;
    while (q--) {
        std::cin >> c >> d;

        std::cout << bidirectional_bfs(c, d, adj_vec) << ENDL;
    }

#ifdef DEBUG
    auto time_end = std::chrono::high_resolution_clock::now();

    auto time_elapsed = std::chrono::duration_cast<std::chrono::milliseconds>(time_end - time_begin).count();
    std::cout << "Time elapsed: " << time_elapsed << "ms" << ENDL;
#endif // DEBUG

    return 0;
}
