#include "playlists.h"

#include <vector>
#include <random>
#include <unordered_map>
#include <list>

namespace {
template<class T>
using vec = std::vector<T>;

template<class T>
using mat = vec<vec<T>>;

template<class K, class V>
using umap = std::unordered_map<K, V>;

using playlist = std::pair<int, umap<int, int>>;

static std::mt19937 random_gen(42);

static std::list<playlist> playlist_set;

static int song_amt;

auto random_int(int min_val, int max_val) {
    return std::uniform_int_distribution<>(min_val, max_val - 1)(::random_gen);
}
}

mat<int> makePlaylists(int n, int k, int s, double p) {
    auto res = mat<int>(n, vec<int>(k));
    ::playlist_set = std::list<playlist>(n);

    ::song_amt = k;

    for (auto& playlist : res) {
        for (auto& song : playlist) {
            song = random_int(0, s);
        }
    }

    auto it = ::playlist_set.begin();
    for (auto i = 0; it != ::playlist_set.end(); ++i, ++it) {
        it->first = i;
        for (auto& song : res[i]) {
            ++it->second[song];
        }
    }

    return res;
}

int guessPlaylist() {
    auto possible_playlists = std::list<playlist>(::playlist_set.begin(), ::playlist_set.end());
    auto curr_playlist = umap<int, int>();
    auto songs_played = 0;

    while (possible_playlists.size() > 1 && songs_played < ::song_amt) {
        auto curr_song = nextSong();
        ++songs_played;

        auto curr_count = ++curr_playlist[curr_song];

        for (auto it = possible_playlists.begin(); it != possible_playlists.end(); ) {
            if (it->second[curr_song] < curr_count) {
                it = possible_playlists.erase(it);
            } else {
                ++it;
            }
        }
    }

    return possible_playlists.front().first;
}
