#include <iostream>
#include <vector>
#include <queue>
#include <algorithm>
#include <set>
#include <cmath>
#include <utility>
#include <tuple>
using namespace std;

const int MAXN = 10004;
const int SQRT = 102;
const int INF = 1 << 20;

int n, m, q;
vector<int> g[MAXN];
int type[MAXN], dist[MAXN], idx[MAXN];

vector<int> black, white;
int bw[SQRT][MAXN];

void Init()
{
    ios :: sync_with_stdio(false);
    cin.tie(NULL); cout.tie(NULL);

    cin >> n >> m;
    for (int i = 1; i <= m; ++ i)
    {
        int a, b;
        cin >> a >> b;
        g[a].push_back(b);
        g[b].push_back(a);
    }

    for (int i = 0; i < n; ++ i)
    {
        dist[i] = INF;
        sort(g[i].begin(), g[i].end());
        if (g[i].size() < n / 2)
        {
            idx[i] = white.size();
            white.push_back(i);
            type[i] = 1;
        }
        else
        {
            idx[i] = black.size();
            black.push_back(i);
            type[i] = 2;
        }
    }
}

int isConnected(int u, int v)
{
    if (u == v) return 1;
    int l = 0, r = g[u].size() - 1;
    while (l <= r)
    {
        int mid = (l + r) / 2;
        if (g[u][mid] == v) return 0; // direct edge is missing
        if (g[u][mid] < v)
        {
            l = mid + 1;
        }
        else if (g[u][mid] > v)
        {
            r = mid - 1;
        }
    }
    return 1;
}

int checkWhites(int u, int v)
{
    if (u == v) return 0;
    if (isConnected(u, v) == 1) return 1;
    else return 2;
}

tuple<int, vector<int>, vector<int>> findFL(int s)
{
    vector<int> flb, flw, vis;
    set<int> flw_set;
    queue<int> q;
    q.push(s);
    dist[s] = 0;
    vis.push_back(s);

    int lev = INF;
    while (!q.empty())
    {
        int u = q.front();
        q.pop();

        if (dist[u] > lev) break; // skip black added before finding the level

        int pos = 0;
        for (int v = 0; v < n; ++ v)
        {
            if (pos != g[u].size() and v == g[u][pos])
            {
                pos++;
                continue;
            }

            if (type[v] == 1 and lev == INF)
            {
                lev = dist[u];
                flw_set.insert(v);
            }
            if (type[v] == 1 and dist[u] == lev)
            {
                flw_set.insert(v);
            }

            if (type[v] == 2 and dist[v] == INF and dist[u] < lev)
            {
                dist[v] = dist[u] + 1;
                q.push(v);
                vis.push_back(v);
            }
        }

        if (type[u] == 2 and dist[u] == lev) flb.push_back(u); // add first level black adjacent to white
    }

    for (int ele : flw_set)
    {
        flw.push_back(ele);
    }

    for (int idx : vis)
    {
        dist[idx] = INF;
    }

    return make_tuple(lev, flw, flb);
}

void findBTW()
{
    for (int i = 0; i < black.size(); ++ i)
    {
        auto res = findFL(i);
        int lev = get<0>(res);
        auto flw = get<1>(res);
        auto flb = get<2>(res);

        for (int ele : flw)
        {
            bw[idx[black[i]]][idx[ele]] = lev + 1;
        }

        int pos = 0;
        for (int j = 0; j < n; ++ j)
        {
            if (pos != flw.size() and flw[pos] == j)
            {
                pos++;
                continue;
            }

            // not first level white
            if (g[j].size() <= n - flb.size()) // certainly have an edge to flb
            {
                bw[idx[black[i]]][j] = lev + 2;
            }
            else // this is N*sqrt(N)*log(N)
            {
                for (int ele : flw)
            }

        }

    }
}

int bfsBlackWhite(int s, int t)
{
    queue<int> q;
    q.push(s);

    vector<int> vis;
    vis.push_back(s);
    dist[s] = 0;

    int path = INF;
    while (!q.empty())
    {
        int u = q.front();
        q.pop();

        int pos = 0;
        for (int v = 0; v < n; ++ v)
        {
            if (pos != g[u].size() and v == g[u][pos])
            {
                pos++;
                continue;
            }
            if (type[v] == 1)
            {
                path = min(path, dist[u] + 1 + checkWhites(v, t))
            }
            if (type[v] == 1 and dist[v] == INF)
            {
                dist[v] = dist[u] + 1;
                q.push(v);
                vis.push_back(v);
            }
        }
    }

    for (int idx : vis)
    {
        dist[idx] = INF;
    }

    if (path == INF) return -1;
    else return path;
}

int main()
{
    Init();

    cin >> q;
    for (int i = 1; i <= q; ++ i)
    {
        int c, d;
        cin >> c >> d;

        if (c == d)
        {
            cout << "0\n";
            continue;
        }

        if (type[c] == 1 and type[d] == 1)
        {
            cout << checkWhites(c, d) << '\n';
        }
        else if (type[c] != type[d])
        {
            if (type[c] == 1) swap(c, d);
            cout << bfsBlack(c, d) << '\n';
        }
    }

    return 0;
}
