#include <iostream>
#include <cstring>
#include <random>
#include <bitset>
#include <queue>
#include <set>
#include <map>

using namespace std;

mt19937 rnd(22);
const int MAXN = 705 + 5;

int n;
bitset <MAXN> isRem[MAXN];
bitset <MAXN> isActive[MAXN];
set <pair <int, int>> allRemoved;

bool isBlocked[MAXN];

int solveEz(int u, int v)
{
    if(u==v) return 0;
    if(allRemoved.count({u, v})==false)
        return 1;

    if((isActive[u]&isActive[v]).count()>MAXN-n)
        return 2;

    bool found = false;
    for(int iter = 0;iter<100;iter++)
    {
        int x = rnd()%n;
        int y = rnd()%n;
        if(isRem[x][y]==1) continue;

        if(isActive[u][x]==1 && isActive[y][v]==1)
        {
            found = true;
            break;
        }
    }
    if(found==true)
        return 3;

    return -1;
}

int dist[MAXN];
map <pair <int, int>, int> answered;

int bfs(int startNode, int endNode)
{
    queue <int> q;
    memset(dist, -1, sizeof(dist));

    q.push(startNode);
    dist[startNode] = 0;

    while(q.empty()==false)
    {
        int x = q.front();
        q.pop();

        answered[{startNode, x}] = dist[x];
        answered[{x, startNode}] = dist[x];

        if(x==endNode)
            return dist[x];

        for(int y = 0;y<n;y++)
        {
            if(dist[y]==-1 && isRem[x][y]==0)
            {
                dist[y] = dist[x] + 1;
                q.push(y);
            }
        }
    }

    return -1;
}

int answerQuery(int u, int v)
{
    if(answered.count({u, v})==true)
        return answered[{u, v}];
    if(isBlocked[u]==true || isBlocked[v]==true)
        return -1;

    int ezAns = solveEz(u, v);
    if(ezAns!=-1) return ezAns;

    return bfs(u, v);
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    int m;
    cin >> n >> m;

    for(int i = 0;i<n;i++)
        isActive[i].flip();

    for(int i = 0;i<m;i++)
    {
        int x, y;
        cin >> x >> y;

        isRem[x][y] = 1;
        isRem[y][x] = 1;
        isActive[x][y] = 0;
        isActive[y][x] = 0;

        allRemoved.insert({x, y});
        allRemoved.insert({y, x});
    }

    for(int x = 0;x<n;x++)
    {
        if(isRem[x].count()==n-1)
            isBlocked[x] = true;
    }

    int Q;
    cin >> Q;

    while(Q--)
    {
        int u, v;
        cin >> u >> v;
        if(u>v) swap(u, v);

        int ans = answerQuery(u, v);
        answered[{u, v}] = ans;
        answered[{v, u}] = ans;

        cout << ans << '\n';
    }
}
