#include <iostream>
#include <cstring>
#include <random>
#include <bitset>
#include <queue>
#include <set>
#include <map>

using namespace std;

mt19937 rnd(22);
const int MAXN = 705 + 5;

int n;
bitset <MAXN> isRem[MAXN];
bitset <MAXN> isActive[MAXN];
set <pair <int, int>> allRemoved;

bool isBlocked[MAXN];
vector <int> graph[MAXN];
vector <int> invGraph[MAXN];

int solveEz(int u, int v)
{
    if(u==v) return 0;
    if(allRemoved.count({u, v})==false)
        return 1;

    for(int x = 0;x<n;x++)
    {
        if(isActive[u][x]==true && isActive[x][v]==true)
            return 2;
    }

    bool found = false;
    for(int iter = 0;iter<100;iter++)
    {
        int x = rnd()%n;
        int y = rnd()%n;
        if(isRem[x][y]==1) continue;

        if(isActive[u][x]==1 && isActive[y][v]==1)
        {
            found = true;
            break;
        }
    }
    if(found==true)
        return 3;

    return -1;
}

int dist[MAXN];
map <pair <int, int>, int> answered;

int bfs(int startNode, int endNode)
{
    queue <int> q;
    memset(dist, -1, sizeof(dist));

    q.push(startNode);
    dist[startNode] = 0;

    while(q.empty()==false)
    {
        int x = q.front();
        q.pop();

        answered[{startNode, x}] = dist[x];
        answered[{x, startNode}] = dist[x];

        if(x==endNode)
            return dist[x];

        for(int y: invGraph[x])
        {
            if(dist[y]==-1)
            {
                dist[y] = dist[x] + 1;
                q.push(y);
            }
        }
    }

    return -1;
}

int depth[MAXN], dp[MAXN];
set <pair <int, int>> s;

void dfs(int x, int last, int level)
{
    depth[x] = level;
    //for(int i = 0;i<level;i++) cout << " ";
    //cout << x << '\n';

    vector <pair <int, int>> removed;
    for(int y: graph[x])
    {
        if(s.count({dp[y], y})==true)
        {
            removed.push_back({dp[y], y});
            s.erase({dp[y], y});
        }
    }

    if(s.empty()==false)
        dp[x] = s.begin()->first + 1;
    else if(last!=-1)
        dp[x] = MAXN;

    for(pair <int, int> p: removed)
        s.insert(p);

    s.insert({dp[x], x});
    for(int y: graph[x])
    {
        if(depth[y]!=-1) continue;
        dfs(y, x, level+1);
    }
    s.erase({dp[x], x});
}

int solve1(int u, int v)
{
    memset(depth, -1, sizeof(depth));
    memset(dp, 0, sizeof(dp));

    //cout << " -> " << u << '\n';
    for(int i = 0;i<graph[u].size();i++)
    {
        if(graph[u][i]==v)
        {
            swap(graph[u][i], graph[u][0]);
            break;
        }
    }

    dp[u] = 0;
    dfs(u, -1, 0);

    if(depth[v]!=-1)
    {
        int ans = dp[v];

        //cout << "dp[v] = " << dp[v] << '\n';
        for(int x = 0;x<n;x++)
        {
            //cout << x << " -> " << dp[x] << '\n';
        }

        for(int x = 0;x<n;x++)
        {
            if(x==v) continue;

            if(allRemoved.count({x, v})==false)
                ans = min(ans, dp[x] + 1);
        }

        return ans;
    }
    else
    {

    }

    return 69;
}

int answerQuery(int u, int v)
{
    if(answered.count({u, v})==true)
        return answered[{u, v}];
    if(isBlocked[u]==true || isBlocked[v]==true)
        return -1;

    int ezAns = solveEz(u, v);
    if(ezAns!=-1) return ezAns;

    return bfs(u, v);
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    int m;
    cin >> n >> m;

    for(int i = 0;i<n;i++)
        isActive[i].flip();

    for(int i = 0;i<m;i++)
    {
        int x, y;
        cin >> x >> y;

        isRem[x][y] = 1;
        isRem[y][x] = 1;
        isActive[x][y] = 0;
        isActive[y][x] = 0;

        allRemoved.insert({x, y});
        allRemoved.insert({y, x});
        graph[x].push_back(y);
        graph[y].push_back(x);
    }

    for(int x = 0;x<n;x++)
    {
        for(int y = 0;y<n;y++)
        {
            if(isActive[x][y]==true)
                invGraph[x].push_back(y);
        }
    }

    for(int x = 0;x<n;x++)
    {
        if(isRem[x].count()==n-1)
            isBlocked[x] = true;
    }

    int Q;
    cin >> Q;

    while(Q--)
    {
        int u, v;
        cin >> u >> v;
        if(u>v) swap(u, v);

        int ans = answerQuery(u, v);
        answered[{u, v}] = ans;
        answered[{v, u}] = ans;

        cout << ans << '\n';
    }
}
