#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x << endl

using namespace std;

const int INF = 1e9;

struct Query {
    int from;
    int to;
    int id;
    int answer;

    Query(int from, int to, int id) {
        this->from = from;
        this->to = to;
        this->id = id;
        this->answer = -1;
    }
};

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n, m;
vector<vector<bool>> edgeMatrix;

void read() {
    cin >> n >> m;

    edgeMatrix.resize(n, vector<bool>(n, true));
    for (int i = 0; i < m; i++) {
        int u, v;
        cin >> u >> v;

        edgeMatrix[u][v] = edgeMatrix[v][u] = false;
    }
}

vector<vector<int>> adjList;
vector<int> dist;

void bfs(int from) {
    fill(dist.begin(), dist.end(), INF);
    queue<int> q;

    dist[from] = 0;
    q.emplace(from);

    while (!q.empty()) {
        int now = q.front();
        q.pop();

        for (int adjNode : adjList[now]) {
            if (dist[adjNode] == INF) {
                dist[adjNode] = dist[now] + 1;
                q.emplace(adjNode);
            }
        }
    }
}

vector<int> compID;
vector<bool> visited;

void dfs(int node) {
    visited[node] = true;

    for (int adjNode : adjList[node]) {
        if (!visited[adjNode]) {
            compID[adjNode] = compID[node];
            dfs(adjNode);
        }
    }
}

bool normalOrderComp(const Query &lhs, const Query &rhs) {
    return lhs.id < rhs.id;
}

bool cIDComp(const Query &lhs, const Query &rhs) {
    return compID[lhs.from] < compID[rhs.from];
}

void solve() {
    adjList.resize(n);
    for (int i = 0; i < n; i++) {
        for (int j = i + 1; j < n; j++) {
            if (edgeMatrix[i][j]) {
                adjList[i].emplace_back(j);
                adjList[j].emplace_back(i);
            }
        }
    }

    compID.resize(n);
    visited.resize(n, false);
    int cID = 0;
    for (int i = 0; i < n; i++) {
        if (!visited[i]) {
            compID[i] = cID++;
            dfs(i);
        }
    }

    int q;
    cin >> q;

    vector<Query> queries;
    queries.reserve(q);
    for (int i = 0; i < q; i++) {
        int from, to;
        cin >> from >> to;

        queries.emplace_back(from, to, i);
    }

    sort(queries.begin(), queries.end(), cIDComp);
    dist.resize(n);

    int prevComputed = -1;
    for (int i = 0; i < q; i++) {
        if (compID[queries[i].from] != compID[queries[i].to]) {
            continue;
        }

        if (queries[i].from != prevComputed) {
            bfs(queries[i].from);
        }

        queries[i].answer = dist[queries[i].to];

        prevComputed = queries[i].from;
    }

    sort(queries.begin(), queries.end(), normalOrderComp);

    for (int i = 0; i < q; i++) {
        cout << queries[i].answer << endl;
    }
}

int main() {
    setIO();

    read();

    solve();
}
