#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x << endl

using namespace std;

const int INF = 1e9;

struct Query {
    int from;
    int to;
    int id;
    int answer;

    Query(int from, int to, int id) {
        this->from = from;
        this->to = to;
        this->id = id;
        this->answer = -1;
    }
};

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n, m;
vector<vector<bool>> edgeMatrix;

void read() {
    cin >> n >> m;

    edgeMatrix.resize(n, vector<bool>(n, true));
    for (int i = 0; i < m; i++) {
        int u, v;
        cin >> u >> v;

        edgeMatrix[u][v] = edgeMatrix[v][u] = false;
    }
}

vector<vector<int>> adjList;

vector<int> compID;
vector<bool> visited;

void dfs(int node) {
    visited[node] = true;

    for (int adjNode : adjList[node]) {
        if (!visited[adjNode]) {
            compID[adjNode] = compID[node];
            dfs(adjNode);
        }
    }
}

vector<vector<int>> dist;

void computeDist(int cID) {
    vector<int> nodes;
    for (int i = 0; i < n; i++) {
        if (compID[i] == cID) {
            nodes.emplace_back(i);
        }
    }

    for (int i = 0; i < n; i++) {
        fill(dist[i].begin(), dist[i].end(), INF);
        dist[i][i] = 0;
    }

    for (int i = 0; i < (int)nodes.size(); i++) {
        for (int j = i + 1; j < (int)nodes.size(); j++) {
            if (edgeMatrix[nodes[i]][nodes[j]]) {
                dist[nodes[i]][nodes[j]] = 1;
                dist[nodes[j]][nodes[i]] = 1;
            }
        }
    }

    for (int k = 0; k < n; k++) {
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                dist[i][j] = min(dist[i][j], dist[i][k] + dist[k][j]);
            }
        }
    }
}

bool normalOrderComp(const Query &lhs, const Query &rhs) {
    return lhs.id < rhs.id;
}

bool cIDComp(const Query &lhs, const Query &rhs) {
    return compID[lhs.from] < compID[rhs.from];
}

void solve() {
    adjList.resize(n);
    for (int i = 0; i < n; i++) {
        for (int j = i + 1; j < n; j++) {
            if (edgeMatrix[i][j]) {
                adjList[i].emplace_back(j);
                adjList[j].emplace_back(i);
            }
        }
    }

    compID.resize(n);
    visited.resize(n, false);
    int cID = 0;
    for (int i = 0; i < n; i++) {
        if (!visited[i]) {
            compID[i] = cID++;
            dfs(i);
        }
    }

    int q;
    cin >> q;

    vector<Query> queries;
    queries.reserve(q);
    for (int i = 0; i < q; i++) {
        int from, to;
        cin >> from >> to;

        queries.emplace_back(from, to, i);
    }

    sort(queries.begin(), queries.end(), cIDComp);;
    dist.resize(n, vector<int>(n));

    int prevCompID = -1;
    for (int i = 0; i < q; i++) {
        if (compID[queries[i].from] != compID[queries[i].to]) {
            continue;
        }

        int cID = compID[queries[i].from];
        if (cID != prevCompID) {
            computeDist(cID);
        }

        queries[i].answer = dist[queries[i].from][queries[i].to];
        prevCompID = cID;
    }

    sort(queries.begin(), queries.end(), normalOrderComp);

    for (int i = 0; i < q; i++) {
        cout << queries[i].answer << endl;
    }
}

int main() {
    setIO();

    read();

    solve();
}
