#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x << endl

using namespace std;

const int INF = 1e9;

struct Query {
    int from;
    int to;
    int id;
    int answer;

    Query(int from, int to, int id) {
        this->from = from;
        this->to = to;
        this->id = id;
        this->answer = -1;
    }
};

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n, m;
vector<vector<bool>> edgeMatrix;

void read() {
    cin >> n >> m;

    edgeMatrix.resize(n, vector<bool>(n, true));
    for (int i = 0; i < m; i++) {
        int u, v;
        cin >> u >> v;

        edgeMatrix[u][v] = edgeMatrix[v][u] = false;
    }
}

vector<vector<int>> adjList;
vector<vector<int>> dist;

void bfs(int node) {
    queue<int> q;

    q.emplace(node);
    dist[node][node] = 0;

    while (!q.empty()) {
        int now = q.front();
        q.pop();

        for (int adjNode : adjList[now]) {
            if (dist[node][adjNode] == INF) {
                dist[node][adjNode] = dist[node][now] + 1;
                q.emplace(adjNode);
            }
        }
    }
}

bool normalOrderComp(const Query &lhs, const Query &rhs) {
    return lhs.id < rhs.id;
}

bool cIDComp(const Query &lhs, const Query &rhs) {
    return lhs.from < rhs.from;
}

vector<int> compID;

void dfs(int node) {
    for (int adjNode : adjList[node]) {
        if (compID[adjNode] == -1) {
            compID[adjNode] = compID[node];
            dfs(adjNode);
        }
    }
}

void solve() {
    adjList.resize(n);
    for (int i = 0; i < n; i++) {
        for (int j = i + 1; j < n; j++) {
            if (edgeMatrix[i][j]) {
                adjList[i].emplace_back(j);
                adjList[j].emplace_back(i);
            }
        }
    }

    dist.resize(n, vector<int>(n, INF));

    compID.resize(n, -1);
    int cID = 0;
    for (int i = 0; i < n; i++) {
        if (compID[i] == -1) {
            compID[i] = cID++;
            dfs(i);
        }
    }

    int q;
    cin >> q;

    vector<Query> queries;
    queries.reserve(q);
    for (int i = 0; i < q; i++) {
        int from, to;
        cin >> from >> to;

        queries.emplace_back(from, to, i);
    }

    sort(queries.begin(), queries.end(), cIDComp);

    for (int i = 0; i < q; i++) {
        int u = queries[i].from;
        int v = queries[i].to;

        if (compID[u] != compID[v]) {
            continue;
        }

        if (dist[u][u] == INF && dist[v][v] < INF) {
            swap(u, v);
        }

        if (dist[u][u] == INF) {
            bfs(u);
        }

        if (dist[u][v] < INF) {
            queries[i].answer = dist[u][v];
        }
    }

    sort(queries.begin(), queries.end(), normalOrderComp);

    for (int i = 0; i < q; i++) {
        cout << queries[i].answer << endl;
    }
}

int main() {
    setIO();

    read();

    solve();
}
